﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WpfCosa
{
    class CellProcessor
    {
        private Window1 outputWindow = null;
        private Int32 tick = 0;
        private Boolean started = false;
        private Boolean debugStep = false;
        private CosaApplication app = null;

        public CellProcessor(Window1 win): base()
        {
            outputWindow = win;
        }

        public void start(Boolean step)
        {
            if (started) return;
            started = true;
            debugStep = step;

            if (app == null) return;
            runApplication();
        }

        private void runApplication()
        {
            if (!app.isStarted())
            {
                app.start();
            }

            while (started && app.isStarted())
            {
                nextTick();
                app.run(tick);
                if (debugStep)
                {
                    started = false;
                }
            }

            if (!app.isStarted())
            {
                unloadApplication(app);
            }
        }

        public void stop()
        {
            started = false;
            unloadApplication(app);
        }

        public void nextTick()
        {
            tick++;
            outputWindow.writeToConsole("Processing tick #" + tick.ToString());
        }

        public Int32 getCurrentTick()
        {
            return tick;
        }

        public void loadApplication(CosaApplication ca)
        {
            app = ca;
            outputWindow.displayMessage("Loaded application: " + app.getTitle());
            runApplication();
        }

        public void unloadApplication(CosaApplication ca)
        {
            if (app == null) return;
            outputWindow.displayMessage("Unloaded application: " + app.getTitle());
            app = null;
        }

        public Boolean isStarted()
        {
            return started;
        }
    }
}
