﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WpfCosa
{
    class DataValue
    {
        private String valueType = "";
        private String valueName = "";
        private String value = null;

        /*
         * Creates a new data value instance from command text
         * Format: {valueType} {valueName} {defaultValue}
         *    where valueType is one of the following: int, string, double, boolean
         *    and valueName must use a valid name
         *    defaultValue is optional
         */
        static public DataValue newInstance(String line)
        {
            DataValue dataValue = null;
            Parser parser = new Parser(line);

            // get line properties
            String valueType = parser.nextToken();
            String valueName = parser.nextToken();
            String defaultValue = parser.nextToken();

            // validate -- throw various exceptions if bad (todo)
            Boolean validType = validateType(valueType);
            Boolean validName = validateName(valueName);
 
            // create if valid
            if (validType && validName)
            {
                dataValue = new DataValue(valueType, valueName);

                // initialize with the default value if specified
                if (defaultValue.Length > 0)
                {
                    switch (valueType)
                    {
                        case "int":
                            dataValue.set(Int32.Parse(defaultValue));
                            break;
                        case "double":
                            dataValue.set(Double.Parse(defaultValue));
                            break;
                        case "string":
                            dataValue.set(defaultValue);
                            break;
                        case "boolean":
                            dataValue.set(Boolean.Parse(defaultValue));
                            break;
                    }
                }
            }

            return dataValue;
        }

        // validates the value type as valid
        // rule: must be either int, boolean, string, double (case sensitive)
        static private Boolean validateType(String valueType)
        {
            String[] validTypes = new String[] { "int", "double", "string", "boolean" };

            Boolean valid = false;
            foreach (String type in validTypes)
            {
                valid |= valueType == type;
            }

            return valid;
        }

        // returns whether or not value name is valid
        // rule: must not be undefined
        static private Boolean validateName(String valueName)
        {
            Boolean valid = true;

            valid &= valueName.Length > 0;

            return valid;
        }

        // private constructor -- want construction line interface
        private DataValue(String type, String name)
            : base()
        {
            valueType = type;
            valueName = name;
        }

        // returns the value name
        public String getName()
        {
            return valueName;
        }

        // returns the value type
        public String getValueType()
        {
            return valueType;
        }

        // sets the current value -- must have a value type of int
        public void set(Int32 iVal)
        {
            if (valueType == "int")
            {
                value = iVal.ToString();
            }
        }

        // sets the current value -- must have a value type of double
        public void set(Double dVal)
        {
            if (valueType == "double")
            {
                value = dVal.ToString();
            }
        }

        // sets the current value -- must have a value type of string
        public void set(String sVal)
        {
            if (valueType == "string")
            {
                value = sVal;
            }
        }

        // sets the current value -- must have a value type of boolean
        public void set(Boolean bVal)
        {
            if (valueType == "boolean")
            {
                value = bVal.ToString();
            }
        }

        // returns value as an int value
        public Int32 getInt()
        {
            return Convert.ToInt32(value);
        }

        // returns value as a double value
        public Double getDouble()
        {
            return Convert.ToDouble(value);
        }

        // returns value as a string value
        public String getString()
        {
            return value;
        }

        // returns value as a boolean value
        public Boolean getBoolean()
        {
            return Convert.ToBoolean(value);
        }

    }
}
