﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace WpfCosa
{
    class ExternalApplication : CosaApplication
    {
        private String sourceFileName = "";
        private Dictionary<String,List<String>> commands;
 
        public ExternalApplication(CellProcessor cp, Int32 tick, String fileName)
            : base(cp, tick)
        {
            sourceFileName = fileName;

            commands = new Dictionary<String, List<String>>();
            commands.Add("init", new List<String>());
            commands.Add("start", new List<String>());
            commands.Add("stop", new List<String>());
            commands.Add("sensor", new List<String>());

            readFile();
        }

        public override String getTitle()
        {
            Int32 pos = sourceFileName.LastIndexOf("\\");
            if (pos < 0) {
                pos = 0;
            }

            return sourceFileName.Substring(pos);
        }

        protected override void init()
        {
            base.init();

            List<String> commandList = commands["init"];
            foreach (String cmd in commandList) {
                initValue(cmd);
            }
        }

        protected override void stop()
        {
            List<String> commandList = commands["stop"];
            foreach (String cmd in commandList)
            {
                runCommand(cmd);
            }

            base.stop();
        }

        protected override void doEffectors(Int32 currentTick)
        {
            List<String> commandList = commands["start"];
            foreach (String cmd in commandList)
            {
                runCommand(cmd);
            }
        }

        protected override void doSensors(Int32 currentTick)
        {
            List<String> commandList = commands["sensor"];
            foreach (String cmd in commandList)
            {
                runCommand(cmd);
            }
        }

        private void readFile() {
            StreamReader reader = new StreamReader(sourceFileName);
            String line = "";
            String section = "";
            try
            {
                do
                {
                    line = nextLine(reader);
                    switch (line)
                    {
                        case "[init]":
                            section = "init";
                            break;
                        case "[start]":
                            section = "start";
                            break;
                        case "[stop]":
                            section = "stop";
                            break;
                        case "[sensor]":
                            section = "sensor";
                            break;
                        default:
                            if (line.Length > 0)
                            {
                                addCommand(section, line);
                            }

                            break;
                    }
                }
                while (reader.Peek() != -1);
            }
            catch
            {
            }
            finally
            {
                reader.Close();
            }
        }

        private void addCommand(String section, String line)
        {
            List<String> commandList = commands[section];
            if (commandList != null)
            {
                commandList.Add(line);
            }
        }

        private String nextLine(StreamReader reader)
        {
            StringBuilder buffer = new StringBuilder();
            do
            {
                if (buffer.Length > 0) buffer.Append(" ");
                buffer.Append(reader.ReadLine().Trim());
            }
            while (!buffer.ToString().EndsWith(";") 
                && !isSectionTag(buffer.ToString()) 
                && reader.Peek() != -1);

            // strip terminator
            if (buffer.ToString().EndsWith(";"))
            {
                buffer.Remove(buffer.Length-1, 1);
            }

            return buffer.ToString();
        }

        private Boolean isSectionTag(String line)
        {
            Boolean tag = false;
            String[] sectionTags = new String[] { "[init]", "[start]", "[sensor]", "[stop]" };

            foreach (String sTag in sectionTags)
            {
                tag |= (line == sTag);
            }

            return tag;
        }
    }
}
