﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Forms;

namespace WpfCosa
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        //private String appPath = "C:\\projects\\CSharp\\Test\\WpfCosa\\SampleFiles\\";
        private CellProcessor processor;

        public Window1()
        {
            InitializeComponent();
            processor = new CellProcessor(this);
        }

        private void startButton_Click(object sender, RoutedEventArgs e)
        {
            if (stepCheck.IsChecked == true)
            {
                nextButton.IsEnabled = true;
            }
            stepCheck.IsEnabled = false;
            stopButton.IsEnabled = true;
            startButton.IsEnabled = false;

            startApplication(stepCheck.IsChecked == true);

            if (!processor.isStarted())
            {
                stopButton_Click(sender, e);
            }
        }

        private void stopButton_Click(object sender, RoutedEventArgs e)
        {
            stopApplication();

            startButton.IsEnabled = true;
            stepCheck.IsEnabled = true;
            nextButton.IsEnabled = false;
            stopButton.IsEnabled = false;
        }

        private void startApplication(bool step)
        {
            processor.start(step);
            writeToConsole("Processor started!");
        }

        private void stopApplication()
        {
            processor.stop();
            writeToConsole("Processor stopped!");
        }

        public void writeToConsole(String text)
        {
            Console.WriteLine(text);
        }

        public void displayMessage(String text) {
            System.Windows.MessageBox.Show(text);
        }

        private void loadButton_Click(object sender, RoutedEventArgs e)
        {
            // open file based on file selected
            String fileName = fileNameText.Text;
            Int32 currentTick = processor.getCurrentTick();
            CosaApplication app = null;

            if (fileName.Length == 0) 
            {
                displayMessage("No file selected.");
            }
            else
            {
                app = new ExternalApplication(processor, currentTick, fileName);
            }

            processor.loadApplication(app);
        }

        private void nextButton_Click(object sender, RoutedEventArgs e)
        {
            processor.start(true);
        }

        private void browseButton_Click(object sender, RoutedEventArgs e)
        {
            // show file open dialog
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.InitialDirectory = System.Windows.Forms.Application.StartupPath;
            openFileDialog.Filter = "COSA Text Files (*.txt)|*.txt|All files (*.*)|*.*";
            openFileDialog.FilterIndex = 1;
            openFileDialog.RestoreDirectory = true;

            DialogResult result = openFileDialog.ShowDialog();
            if (result == System.Windows.Forms.DialogResult.OK)
            {
                fileNameText.Text = openFileDialog.FileName;
            }  

        }
    }
}
